table, td {
  border: 1px solid black;
  padding: 4px; }

.odd {
  background-color: #eeeeff; }

.even {
  background-color: #ffeeee; }

.detailed-info {
  display: block; }

.action.active {
  text-decoration: none;
  color: black;
  font-weight: bold; }

#select-version {
  background-color: #eeffee;
  position: absolute;
  border: 3px solid #009900;
  padding: 2em;
  left: 30em;
  top: 1em;
  bottom: 1em;
  right: 2em; }
