<html>
<head>
  <title>Pax Daemonica</title>
  <script type="text/javascript" src="https://ajax.googleapis.com/ajax/libs/jquery/1.6.1/jquery.min.js"></script>
  <script type="text/javascript" src="/jquery.js"></script>

  ${includes()}
  </head>
<body>
  ${next.body()}
</body>
</html>

<%def name="includes()">
<script>
  <%include file="/webuiapp.js"/>
</script>
<style>
  <%include file="/webuiapp.css"/>
</style>
</%def>